package com.example.pepat.weatherapp;

import android.os.AsyncTask;
import android.util.Log;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;

import com.google.android.gms.location.places.ui.PlaceAutocompleteFragment;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

public class GetCurrentWeather extends AsyncTask<String, Void, ArrayList<String>> {
    private Button tempButton;
    private PlaceAutocompleteFragment textView2;
    private ImageView imageView;
    private TextView textViewLocation;
    private TextView textIcon;
    private ImageView imageWindIcon;
    private String units;


    public GetCurrentWeather(Button tempButton, PlaceAutocompleteFragment textView2, ImageView imageView,
                             TextView textViewLocation, TextView textIcon, ImageView imageWindIcon, String units) {
        this.tempButton = tempButton;
        this.textView2 = textView2;
        this.imageView = imageView;
        this.textViewLocation = textViewLocation;
        this.textIcon = textIcon;
        this.imageWindIcon = imageWindIcon;
        this.units = units;
    }

    @Override
    protected ArrayList<String> doInBackground(String... params) {



        String weather = "UNDEFINED";
        ArrayList<String> data = new ArrayList<String>();
        try {
            URL url = new URL(params[0]);
            HttpURLConnection urlConnection = (HttpURLConnection) url.openConnection();

            InputStream stream = new BufferedInputStream(urlConnection.getInputStream());
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(stream));
            StringBuilder builder = new StringBuilder();

            String inputString;
            while ((inputString = bufferedReader.readLine()) != null) {
                builder.append(inputString);
            }

            JSONObject topLevel = new JSONObject(builder.toString());
            JSONObject main = topLevel.getJSONObject("main");
            weather = String.valueOf(main.getDouble("temp"));

            Double temp1 = main.getDouble("temp");


            NumberFormat formatter = new DecimalFormat("#0");

            String temp = formatter.format(temp1);
            Log.i("temp", temp);

            String name = topLevel.getString("name");

            JSONObject coords = topLevel.getJSONObject("coord");

            String latti = coords.getString("lat");
            String longi = coords.getString("lon");


            data.add(temp);
            data.add(name);
            data.add(latti);
            data.add(longi);

            JSONArray weatherInfo = topLevel.getJSONArray("weather");
            Log.i("weatherInfoCurrent", weatherInfo.toString());
            JSONObject weatherInfoObject = weatherInfo.getJSONObject(0);




            String weatherMain = weatherInfoObject.getString("main");
            String weatherDescription = weatherInfoObject.getString("description");

            data.add(weatherDescription);
            JSONObject wind = topLevel.getJSONObject("wind");
            Double speed = wind.getDouble("speed");
            String windFormat = formatter.format(speed);
            data.add(windFormat);
            try {
                Integer deg = wind.getInt("deg");
                Log.i("tuuli", speed.toString());
                Log.i("kulma", deg.toString());
                data.add(deg.toString());
            }catch (Exception e){

            }
            urlConnection.disconnect();
        } catch (IOException | JSONException e) {
            e.printStackTrace();

        }
        return data;
    }



    @Override
    protected void onPostExecute(ArrayList<String> dataList) {
        String unitsToText;
        String unitsToWind;
        if(units == "metric"){
            unitsToText = "°C";
            unitsToWind = "m/s";
        }
        else{
            unitsToText = "°F";
            unitsToWind = "mi/h";
        }
        try {
            tempButton.setText(dataList.get(0) + unitsToText);
            textView2.setText(dataList.get(1));
            String latti = dataList.get(2);
            double lattiDouble = Double.parseDouble(latti);
            String longi = dataList.get(3);
            double longiDouble = Double.parseDouble(longi);

            NumberFormat formatter = new DecimalFormat("#0.0");

            String lattitude = formatter.format(lattiDouble);
            Log.i("lat", lattitude);


            String longitude = formatter.format(longiDouble);
            Log.i("lon", longitude);

            textViewLocation.setText("(" + lattitude + "N , " + longitude + "E)");

            DateFormat dateFormat = new SimpleDateFormat("HH");
            Date date = new Date();
            Log.i("aika", dateFormat.format(date));

            setIcon(imageView, dataList.get(4), dateFormat.format(date));

            textIcon.setText(dataList.get(5) + unitsToWind);



            if(dataList.size() == 7) {
                Log.i("listan koko currentissa", dataList.size() + "");
                setWindIcon(imageWindIcon, dataList.get(6));
            }

        }catch (IndexOutOfBoundsException e){
            e.printStackTrace();
        }
    }

    protected void setWindIcon(ImageView imageWindIcon, String windDeg){
        int windAngle = Integer.parseInt(windDeg);
        if((0 <= windAngle && windAngle <= 45) || (315 <= windAngle && windAngle <=360)){
            imageWindIcon.setImageResource(R.drawable.tuuli0);
        }
        else if(45 <= windAngle && windAngle <= 135){
            imageWindIcon.setImageResource(R.drawable.tuuli90);
        }
        else if(135 <= windAngle && windAngle <= 225){
            imageWindIcon.setImageResource(R.drawable.tuuli180);
        }
        else{
            imageWindIcon.setImageResource(R.drawable.tuuli270);
        }
    }

    protected void setIcon(ImageView imageViewIcon, String weather, String time){
        String saa;
        Log.i("weatherDescription", weather);
        if(weather.equals("light rain")){
            saa = "sade1";
        }
        else if(weather.equals("moderate rain")){
            saa = "sade2";
        }
        else if(weather.equals("heavy intensite rain")){
            saa = "sade3";
        }
        else if(weather.equals("clear sky")){
            if(time.equals("00") || time.equals("01") || time.equals("02") || time.equals("03") ||
                    time.equals("04") || time.equals("05")){
                saa = "kuu";
            }
            else {
                saa = "aurinko";
            }
        }
        else if(weather.equals("few clouds")){
            if(time.equals("00") || time.equals("01") || time.equals("02") || time.equals("03") ||
                    time.equals("04") || time.equals("05")){
                saa = "puolipilvinen_yo";
            }
            else {
                saa = "puolipilvinen";
            }
        }
        else if(weather.equals("scattered clouds")){
            saa = "pilvi";
        }
        else if(weather.equals("broken clouds")){
            saa = "pilvi";
        }
        else if(weather.equals("overcast clouds")){
            saa = "pilvi";
        }
        else{
            saa = "pilvi";
        }
        Log.i("saa",saa);
        switch (saa){
            case "sade1":
                imageViewIcon.setImageResource(R.drawable.sade1);
                break;
            case "sade2":
                imageViewIcon.setImageResource(R.drawable.sade2);
                break;
            case "sade3":
                imageViewIcon.setImageResource(R.drawable.sade3);
                break;
            case "pilvi":
                imageViewIcon.setImageResource(R.drawable.pilvi);
                break;
            case "aurinko":
                imageViewIcon.setImageResource(R.drawable.aurinko);
                break;
            case "puolipilvinen":
                imageViewIcon.setImageResource(R.drawable.puolipilvinen);
                break;

            case "kuu":
                imageViewIcon.setImageResource(R.drawable.kuu);
                break;
            case "puolipilvinen_yo":
                imageViewIcon.setImageResource(R.drawable.puolipilvinen_yo);
                break;
        }
    }
}
